@@ -45,7 +45,7 @@ class DataLocation(Configuration):
     The path to a flat-file inside an object storage system. When using the Modela API to upload files (through the
     FileService API), Modela will upload the data to a predetermined path based on the Tenant, DataProduct,
     DataProductVersion, and resource type of the resource in relation to the file being uploaded.
-    The path does not need to adhere to this format; you can still pass the path of a file inside a bucket not managed by Modela
+    The path does not need to adhere to this format; you can give the path to a file inside a bucket not managed by Modela
     """
     Table: str = ''
     """ The name of a table inside a database, if applicable """
@@ -55,6 +55,16 @@ class DataLocation(Configuration):
     """ The SQL statement which will be executed to query data from the table specified by Table """
     Topic: str = ''
     """ The name of the streaming topic (currently unsupported) """
+    Url: str = ''
+    """
+    In the case of the location type being WebApi, URL specifies the external location (HTTP or Git) that will be queried
+    and then stored as flat-file by the resource which specifies the DataLocation
+    """
+    ResourceRef: ObjectReference = None
+    """
+    In the case of the location type being Dataset or PublicDataset, ResourceRef references another resource that
+    containing data that will be used as a data source
+    """
 
 
 @datamodel(proto=data_pb.SampleSpec)
@@ -170,7 +180,9 @@ class DataProductSpec(Configuration):
     ServingSiteName: str = 'default-serving-site'
     """ The name of the Serving Site which will be used by default with all Predictor resources """
     Task: TaskType = TaskType.BinaryClassification
-    """ Task denote the machine learning task of the product (classification/regression,etc.) """
+    """ Task specifies the default machine learning task of the product (classification, regression, etc.) """
+    Subtask: SubtaskType = None
+    """ Subtask specifies the default subtask relevant to the primary task (text classification, image object detection, etc.) """
     Description: str = ''
     """ User-provided description of the object """
     DataLocation: DataLocationType = DataLocationType()
@@ -377,8 +389,8 @@ class Column(Configuration):
     """ The threshold skew for skew detection """
     DriftThreshold: float = None
     """ The threshold drift value for model drift detection. """
-    Index: bool = True
-    """ Indicates if the column is an index column """
+    Index: int = 0
+    """ Contain the Index for the column in the schema """
     Fold: bool = False
     """ Indicates if the column holds fold values """
     Weight: bool = False
@@ -397,6 +409,8 @@ class Column(Configuration):
     """ Indicates if the column is an ID column """
     Step: float = 1
     """ The step value if the column values are a sequence of numbers """
+    IndexColumn: bool = False
+    """ Indicates if the column is an index column """
 
 
 @datamodel(proto=data_pb.TimeSeriesSchema)
@@ -415,22 +429,22 @@ class RecommendationSchema(Configuration):
     """ The name of the column that specifies ratings """
 
 
-@datamodel(proto=data_pb.MultiDatasetValidation)
-class MultiDatasetValidation(Configuration):
-    """ MultiDatasetValidation specifies a validation rule between multiple datasets """
-    Type: MultiDatasetValidationRule = None
-    """ The type of validation rule """
+@datamodel(proto=data_pb.MultiDatasetTest)
+class MultiDatasetTest(Configuration):
+    """ MultiDatasetTestName specifies a test between multiple datasets """
+    Type: MultiDatasetTestName = None
+    """ The type of test """
     Datasets: List[str] = field(default_factory=lambda : [])
     """ The list of datasets that will be checked """
     Generated: bool = False
-    """ Indicates if this validation rule was automatically generated by the system """
+    """ Indicates if this test was automatically generated by the system """
 
 
-@datamodel(proto=data_pb.DatasetValidation)
-class DatasetValidation(Configuration):
-    """ DatasetValidation specifies a validation rule for the contents of an entire dataset """
-    Type: DatasetValidationRule = None
-    """ The type of validation rule """
+@datamodel(proto=data_pb.DatasetTest)
+class DatasetTest(Configuration):
+    """ DatasetTestName specifies a test for the contents of an entire dataset """
+    Type: DatasetTestName = None
+    """ The type of test """
     Value: float = 0
     Min: float = 0
     Max: float = 0
@@ -438,14 +452,14 @@ class DatasetValidation(Configuration):
     StrictMin: bool = False
     StrictMax: bool = False
     Generated: bool = False
-    """ Indicates if this validation rule was automatically generated by the system """
+    """ Indicates if this test was automatically generated by the system """
 
 
-@datamodel(proto=data_pb.MultiColumnValidation)
-class MultiColumnValidation(Configuration):
-    """ MultiColumnValidation specifies a validation rule that encompasses multiple columns """
-    Type: MultiColumnValidationRule = None
-    """ The type of validation rule """
+@datamodel(proto=data_pb.MultiColumnTest)
+class MultiColumnTest(Configuration):
+    """ MultiColumnTest specifies a test that encompasses multiple columns """
+    Type: MultiColumnTestName = None
+    """ The type of test  """
     Columns: List[str] = field(default_factory=lambda : [])
     """ The list of columns that will be checked """
     Value: float = 0
@@ -455,14 +469,14 @@ class MultiColumnValidation(Configuration):
     StrictMin: bool = False
     StrictMax: bool = False
     Generated: bool = False
-    """ Indicates if this validation rule was automatically generated by the system """
+    """ Indicates if this test was automatically generated by the system """
 
 
-@datamodel(proto=data_pb.ColumnValidation)
-class ColumnValidation(Configuration):
-    """ ColumnValidation specifies a validation rule for a single column """
-    Type: ColumnValidationRule = None
-    """ The type of validation rule """
+@datamodel(proto=data_pb.ColumnTest)
+class ColumnTest(Configuration):
+    """ ColumnTest specifies a test rule for a single column """
+    Type: ColumnTestName = None
+    """ The type of test """
     Column: str = ''
     """ The name of the column to be validated """
     Value: float = 0
@@ -472,53 +486,53 @@ class ColumnValidation(Configuration):
     StrictMin: bool = False
     StrictMax: bool = False
     Generated: bool = False
-    """ Indicates if this validation rule was automatically generated by the system """
+    """ Indicates if this test was automatically generated by the system """
 
 
-@datamodel(proto=data_pb.FileValidation)
-class FileValidation(Configuration):
-    Type: FileValidationRule = None
+@datamodel(proto=data_pb.FileTest)
+class FileTest(Configuration):
+    Type: FileTestName = None
     BucketName: str = ''
     Column: str = ''
     Value: float = 0
-    """ For validation that need a single value """
+    """ For tests that need a single value """
     Min: float = 0
     Max: float = 0
     ValueSet: List[str] = field(default_factory=lambda : [])
     StrictMin: bool = False
     StrictMax: bool = False
     Generated: bool = False
-    """ Generated indicate weather this validation rule was automatically generated """
+    """ Generated indicate weather this test was automatically generated """
 
 
-@datamodel(proto=data_pb.ValidationSpec)
-class ValidationRules(Configuration):
-    MultiDatasetValidations: List[MultiDatasetValidation] = field(default_factory=lambda : [])
-    DatasetValidations: List[DatasetValidation] = field(default_factory=lambda : [])
-    """ DatasetValidations contains validations for the whole dataset """
-    MultiColumnValidations: List[MultiColumnValidation] = field(default_factory=lambda : [])
-    """ MultiColumnValidations defines validations for multiple columns from the dataset """
-    ColumnValidations: List[ColumnValidation] = field(default_factory=lambda : [])
-    """ ColumnValidations defines assertions for columns from the dataset """
-    FileValidations: List[FileValidation] = field(default_factory=lambda : [])
-    """ FileValidations defines assertions for the contents of the data file """
+@datamodel(proto=data_pb.TestSpec)
+class Tests(Configuration):
+    MultiDatasetTests: List[MultiDatasetTest] = field(default_factory=lambda : [])
+    DatasetTests: List[DatasetTest] = field(default_factory=lambda : [])
+    """ DatasetTests contains tests for the whole dataset """
+    MultiColumnTests: List[MultiColumnTest] = field(default_factory=lambda : [])
+    """ MultiColumnTests defines tests for multiple columns from the dataset """
+    ColumnTests: List[ColumnTest] = field(default_factory=lambda : [])
+    """ ColumnTests defines assertions for columns from the dataset """
+    FileTests: List[FileTest] = field(default_factory=lambda : [])
+    """ FileTests defines assertions for the contents of the data file """
 
 
 MDTimeSeriesSchema = TimeSeriesSchema
 MDRecommendationSchema = RecommendationSchema
-
+MDTests = Tests
 
 @datamodel(proto=data_pb.Schema)
 class Schema(Configuration):
-    """ Schema defines the column-level format and validation rules for data associated with a DataSource """
+    """ Schema defines the column-level format and tests for data associated with a DataSource """
     Columns: List[Column] = field(default_factory=lambda : [])
     """ The collection of columns and their attributes """
     TimeSeriesSchema: MDTimeSeriesSchema = None
     """ The time-series schema, which sets time-series specific parameters """
     RecommendationSchema: MDRecommendationSchema = None
     """ The recommendation schema, which is used for the recommendation ML task """
-    Validation: ValidationRules = ValidationRules()
-    """ The specification for validation rules which will be performed on new Datasets """
+    Tests: MDTests = MDTests()
+    """ The specification for test rules which will be performed on new Datasets """
 
 
 @datamodel(proto=data_pb.RelationshipSpec)
@@ -545,6 +559,17 @@ class CorrelationSetting(Configuration):
     """ The number of top correlations to be included in the correlation results """
 
 
+@datamodel(proto=data_pb.FlatFileFormatSpec)
+class FlatFileFormat(Configuration):
+    """ FlatFileFormatSpec defines the format for incoming flat-files to be parsed """
+    FileType: FlatFileType = FlatFileType.Csv
+    """ The file type of incoming data which uses the DataSource (by default, a CSV file) """
+    Csv: CsvFileFormat = CsvFileFormat()
+    """ The file format for CSV files, if applicable """
+    Excel: ExcelNotebookFormat = ExcelNotebookFormat()
+    """ The file format for Excel files, if applicable """
+
+
 DataSourceSchema = Schema
 
 
@@ -569,19 +594,19 @@ class DataSourceSpec(Configuration):
     """
     Description: str = ''
     """ User-provided description of the object """
-    FileType: FlatFileType = FlatFileType.Csv
-    """ The file type of incoming data which uses the DataSource (by default, a CSV file) """
     Task: TaskType = TaskType.BinaryClassification
     """
-    The default task for Dataset resources created from the DataSource. If null, this will be determined from
-    the default task of the DataProduct which owns the DataSource
+    The default task for Dataset resources created from the Data Source. If null, the task type will default to the
+    the default task type of the Data Product which contains the Data Source
     """
-    Csvfile: CsvFileFormat = CsvFileFormat()
-    """ The file format for CSV files, if applicable """
-    ExcelNotebook: ExcelNotebookFormat = None
-    """ The file format of excel files, if applicable """
+    Subtask: SubtaskType = None
+    """ The machine learning subtask relevant to the primary task (text classification, image object detection, etc.) """
+    Flatfile: FlatFileFormat = FlatFileFormat()
+    """ Flat file spec define the paramter needed to read a flat file. """
     DatasetType: DatasetType = DatasetType.Tabular
     """ The type of dataset; currently, the only supported type is `tabular` """
+    InferredFrom: DataLocation = None
+    """ InferredFrom specifies the location of the data that was used to generate the schema of the Data Source """
 
 
 @datamodel(proto=data_pb.DatasetSpec)
@@ -610,7 +635,7 @@ class DatasetSpec(Configuration):
     DatasourceName: str = ''
     """
     The reference to the Data Source resource which exists in the same Data Product namespace as the object.
-    The Data Source must represent the columns and the task type of the Dataset. The validation rules associated with
+    The Data Source must represent the columns and the task type of the Dataset. The tests associated with
     the Data Source will be validated against the raw data of the Dataset once it is created
     """
     Description: str = ''
@@ -625,7 +650,7 @@ class DatasetSpec(Configuration):
     directly from a database, and must be converted to a flat-file type such as a CSV as a result
     """
     Validate: bool = True
-    """ Indicates if the Dataset should be checked against the validation rules of its Data Source """
+    """ Indicates if the Dataset should be checked against the tests of its Data Source """
     Labeled: bool = True
     """ Indicates if the data is labeled or not """
     Synthetic: bool = False
@@ -645,6 +670,8 @@ class DatasetSpec(Configuration):
     """
     Task: TaskType = None
     """ The machine learning task relevant to the Dataset. This field *must* be the same as the Data Source of the object """
+    Subtask: SubtaskType = None
+    """ The machine learning sub task relevant to the Dataset. This field *must* be the same as the Data Source of the object """
     Notification: NotificationSettings = None
     """ The notification specification that determines which notifiers will receive Alerts generated by the object """
     Correlation: CorrelationSetting = None
@@ -659,6 +686,15 @@ class DatasetSpec(Configuration):
     """
 
 
+@datamodel(proto=data_pb.OutlierStat)
+class OutlierStat(Configuration):
+    Lower: int = 0
+    Upper: int = 0
+    """ The number of outliers above baseline """
+    Percent: float = 0
+    """ Percentage of rows detected as outliers  """
+
+
 @datamodel(proto=data_pb.ColumnStatistics)
 class ColumnStatistics(Configuration):
     """ ColumnStatistics contains statistical parameters for a single feature from a dataset """
@@ -737,8 +773,6 @@ class ColumnStatistics(Configuration):
     """ Indicates if the column is a duplicate of another column """
     Reserved: bool = False
     """ Indicates if the column is reserved and must be a feature included in model training """
-    Outliers: int = 0
-    """ The amount of outliers detected in the columns values """
     Completeness: float = 0
     """ The ratio between non-null and null values in the column """
     DistinctValueCount: float = 0
@@ -749,6 +783,10 @@ class ColumnStatistics(Configuration):
     """ Used for text attributes """
     CorrToTarget: float = 0
     """ Correlation to the target feature """
+    Index: int = 0
+    """ The column index in the dataset """
+    Outliers: OutlierStat = None
+    """ Outlier statistics. """
 
 
 @datamodel(proto=common_pb.ColumnProfile)
@@ -792,13 +830,16 @@ class ColumnProfile(Configuration):
     Constant: bool = False
     Duplicate: bool = False
     Reserved: bool = False
-    Outliers: int = 0
     Completeness: float = 0
     DistinctValueCount: float = 0
     MostFreqValuesRatio: float = 0
     IndexOfPeculiarity: float = 0
     Values: List[str] = field(default_factory=lambda : [])
     CorrToTarget: float = 0
+    OutliersUpper: int = 0
+    OutliersLower: int = 0
+    OutliersPercent: float = 0
+    Index: int = 0
 
 
 @datamodel(proto=common_pb.DatasetProfile)
@@ -866,8 +907,8 @@ class DatasetCondition(Configuration):
     """ A human readable message indicating details about the transition. """
 
 
-@datamodel(proto=data_pb.DataValidationResult)
-class DataValidationResult(Configuration):
+@datamodel(proto=data_pb.DataTestResult)
+class DataTestResult(Configuration):
     Type: str = ''
     Column: str = ''
     Error: str = ''
@@ -894,8 +935,8 @@ class DatasetStatus(ImmutableConfiguration):
     """ Whether or not the data was detected as imbalanced """
     ObservedGeneration: int = 0
     """ ObservedGeneration is the last generation that was acted on """
-    ValidationResults: List[DataValidationResult] = field(default_factory=lambda : [])
-    """ List of validation results which are generated for every validation rule associated with the Dataset's Data Source """
+    TestResults: List[DataTestResult] = field(default_factory=lambda : [])
+    """ List of test results which are generated for every test associated with the Dataset's Data Source """
     LastStudyTime: Time = None
     """ Last time the Dataset was used with a Study """
     FailureReason: StatusError = None
