@@ -1,14 +1,10 @@
-from typing import Optional
+import subprocess
+import socket
+from contextlib import closing
+
+from typing import Callable
 
-import jwt
 from github.com.metaprov.modelaapi.services.account.v1.account_pb2 import AccountLoginRequest
-from grpc import (  # type: ignore
-    UnaryUnaryClientInterceptor,
-    UnaryStreamClientInterceptor,
-    StreamUnaryClientInterceptor,
-    StreamStreamClientInterceptor,
-)
-
 from github.com.metaprov.modelaapi.services.datapipelinerun.v1 import datapipelinerun_pb2_grpc
 from github.com.metaprov.modelaapi.services.datapipeline.v1 import datapipeline_pb2_grpc
 from github.com.metaprov.modelaapi.services.dataproduct.v1 import dataproduct_pb2_grpc
@@ -50,6 +46,7 @@ from github.com.metaprov.modelaapi.services.tenant.v1 import tenant_pb2_grpc
 from github.com.metaprov.modelaapi.services.virtualbucket.v1 import virtualbucket_pb2_grpc
 from github.com.metaprov.modelaapi.services.virtualcluster.v1 import virtualcluster_pb2_grpc
 from github.com.metaprov.modelaapi.services.virtualvolume.v1 import virtualvolume_pb2_grpc
+from github.com.metaprov.modelaapi.services.userroleclass.v1 import userroleclass_pb2_grpc
 from github.com.metaprov.modelaapi.services.meeting.v1 import meeting_pb2_grpc
 from github.com.metaprov.modelaapi.services.postmortem.v1 import postmortem_pb2_grpc
 from github.com.metaprov.modelaapi.services.review.v1 import review_pb2_grpc
@@ -108,6 +105,7 @@ from modela.infra.Tenant import *
 from modela.infra.VirtualBucket import *
 from modela.infra.VirtualCluster import *
 from modela.infra.VirtualVolume import *
+from modela.infra.UserRoleClass import *
 from modela.team.Meeting import *
 from modela.team.PostMortem import *
 from modela.team.Review import *
@@ -123,6 +121,29 @@ from modela.training.NotebookRun import *
 from modela.training.Notebook import *
 from modela.training.Report import *
 from modela.training.Study import *
+from grpc_interceptor import ClientCallDetails, ClientInterceptor
+
+
+class AuthClientInterceptor(ClientInterceptor):
+    def __init__(self, token: str):
+        self.token = token
+
+    def intercept(
+            self,
+            method: Callable,
+            request_or_iterator,
+            call_details: grpc.ClientCallDetails,
+    ):
+        new_details = ClientCallDetails(
+            call_details.method,
+            call_details.timeout,
+            [("authorization", self.token)],
+            call_details.credentials,
+            call_details.wait_for_ready,
+            call_details.compression,
+        )
+
+        return method(request_or_iterator, new_details)
 
 
 class Modela:
@@ -131,40 +152,46 @@ class Modela:
     resources on your cluster. The class provides you with a directory for all resource API clients, and functions to
     fetch or create each resource.
     """
+
     def __init__(
             self,
             host="localhost",
             port=3000,
-            username: str = None,
-            password: str = None,
+            username: str = "admin",
+            password: str = "admin",
             secure=False,
             tls_cert=None,
             tenant="default-tenant",
-            api_token: str = None,
-            interceptors: Optional[
-                List[
-                    Union[
-                        UnaryUnaryClientInterceptor,
-                        UnaryStreamClientInterceptor,
-                        StreamUnaryClientInterceptor,
-                        StreamStreamClientInterceptor,
-                    ]
-                ]
-            ] = None,
+            port_forward=False,
     ):
         """
         Connect to the Modela API gateway.
 
-        :param host:
-        :param port:
-        :param username:
-        :param password:
-        :param secure:
-        :param tls_cert:
-        :param tenant:
-        :param api_token:
+        :param host: The DNS name or IP that hosts the API gateway.
+        :param port: The port which exposes the API gateway.
+        :param username: Username for your Modela account that has permissions to use the specified tenant.
+        :param password: Password for your Modela account.
+        :param secure: If connecting through gRPC ingress, secure must be enabled and a TLS public key must be supplied.
+        :param tls_cert: The TLS public key of the ingress that exposes the API gateway gRPC service.
+        :param tenant: The tenant that Modela SDK objects will utilize by default (default: `default-tenant`)
+        :param port_forward: If enabled, the SDK will attempt to port forward the API gateway using kubectl. Kubectl
+            must be installed and must be connected to a cluster with Modela installed.
         """
+
         self.tenant = tenant
+        if port_forward:
+            with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
+                s.bind(('', 0))
+                s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+                port = s.getsockname()[1]
+
+            self.pf_process = subprocess.Popen(
+                "kubectl port-forward svc/modela-api-gateway %d:8080 -n modela-system" % port,
+                shell=True, stderr=subprocess.STDOUT)
+
+            time.sleep(1 / 4)
+            secure, host = False, "localhost"
+
         if secure:
             with open(tls_cert, 'rb') as f:
                 credentials = grpc.ssl_channel_credentials(f.read())
@@ -176,10 +203,10 @@ class Modela:
             else:
                 self._channel = grpc.insecure_channel(f'{host}')
 
-        if interceptors:
-            self._channel = grpc.intercept_channel(  # type: ignore
-                self._channel, *interceptors
-            )
+        # if interceptors:
+        #     self._channel = grpc.intercept_channel(  # type: ignore
+        #         self._channel, *interceptors
+        #     )
 
         self.__account_stub = account_pb2_grpc.AccountServiceStub(self._channel)
         self.__account_client = AccountClient(self.__account_stub, self)
@@ -187,9 +214,10 @@ class Modela:
         if username is not None:
             login_request = AccountLoginRequest(namespace=tenant, name=username, password=password)
             token = self.__account_stub.Login(login_request).token
-            # jwt.decode(token, algorithms=["HS256"], options={"verify_signature": False}))
-
+            self._channel = grpc.intercept_channel(self._channel, AuthClientInterceptor(token))
 
+        self.__account_stub = account_pb2_grpc.AccountServiceStub(self._channel)
+        self.__account_client = AccountClient(self.__account_stub, self)
 
         self.__fileservice_stub = fileservices_pb2_grpc.FileServicesServiceStub(self._channel)
         self.__fileservice_client = FileService(self.__fileservice_stub)
@@ -311,6 +339,9 @@ class Modela:
         self.__virtualvolume_stub = virtualvolume_pb2_grpc.VirtualVolumeServiceStub(self._channel)
         self.__virtualvolume_client = VirtualVolumeClient(self.__virtualvolume_stub, self)
 
+        self.__userroleclass_stub = userroleclass_pb2_grpc.UserRoleClassServiceStub(self._channel)
+        self.__userroleclass_client = UserRoleClassClient(self.__userroleclass_stub, self)
+
         self.__meeting_stub = meeting_pb2_grpc.MeetingServiceStub(self._channel)
         self.__meeting_client = MeetingClient(self.__meeting_stub, self)
 
@@ -367,22 +398,29 @@ class Modela:
     def DataProducts(self) -> DataProductClient:
         return self.__dataproduct_client
 
-    def DataProduct(self, namespace="", name="", servingsite: str = None,
-                    lab: str = None, task_type: TaskType = TaskType.BinaryClassification,
-                    default_workload: Workload = None,
-                    default_bucket: str = None, notification_setting: NotificationSetting = None) -> DataProduct:
+    def DataProduct(self, namespace="", name="", serving_site: Union[ServingSite, str] = None,
+                    lab: Union[Lab, str] = None,
+                    public: bool = None, task_type: TaskType = None, default_training_workload: Workload = None,
+                    default_serving_workload: Workload = None, default_bucket: str = None,
+                    notification_settings: NotificationSettings = None,
+                    permissions: Permissions = None) -> DataProduct:
         """
         :param namespace: The target namespace of the resource.
         :param name: The name of the resource.
-        :param servingsite: The default Serving Site of the Data Product
-        :param lab: The default Lab of the Data Product
-        :param task_type: The default task type of the Data Product
-        :param default_workload: The default workload of the Data Product
-        :param default_bucket: The default bucket used for all Data Product resources.
-        :param notification_setting: The default notification settings used for all Data Product resources.
+        :param serving_site: The default Serving Site of the Data Product.
+        :param lab: The default Lab of the Data Product.
+        :param public: If enabled, the Data Product will be publicly accessible by all users without permissions.
+        :param task_type: The default task type for child resources of the Data Product.
+        :param default_training_workload: The default workload for training Jobs under the Data Product
+        :param default_serving_workload: The default workload for model serving Jobs under the Data Product.
+        :param default_bucket: The default bucket used for child resources of the Data Product.
+        :param notification_settings: The default notification settings used for child resources of the Data Product.
+        :param permissions: The permission specification that dictates which users can access the resources under
+          the Data Product and what actions they can perform.
         """
-        return DataProduct(MDDataProduct(), self.DataProducts, namespace, name, servingsite, lab, task_type,
-                           default_workload, default_bucket, notification_setting)
+        return DataProduct(MDDataProduct(), self.DataProducts, namespace, name, serving_site, lab, public,
+                           task_type, default_training_workload, default_serving_workload, default_bucket,
+                           notification_settings, permissions)
 
     @property
     def DataSources(self) -> DataSourceClient:
@@ -390,7 +428,7 @@ class Modela:
 
     def DataSource(self, namespace="", name="", version=Resource.DefaultVersion, bucket: str = "default-minio-bucket",
                    infer_file: str = None, infer_dataframe: pandas.DataFrame = None, infer_bytes: bytes = None,
-                   target_column: str = "", file_type: FlatFileType = FlatFileType.Csv,
+                   target_column: str = "", file_type: FlatFileType = None,
                    task_type: TaskType = None, csv_config: CsvFileFormat = None,
                    excel_config: ExcelNotebookFormat = None) -> DataSource:
         """
@@ -406,8 +444,8 @@ class Modela:
             the columns and generate a schema that will be applied to the resource.
         :param target_column: The name of the target column used when training a model. This parameter only has effect
             when data is uploaded to infer a schema.
-        :param file_type: The file type of raw data, used when ingesting a Dataset. Only applicable for flat files.
-            If inferring from a dataframe, the file type will default to CSV.
+        :param file_type: The file type of raw data, used when ingesting a Dataset from a file, or creating a data snapshot
+            from a database source. If inferring from a dataframe, the file type will default to CSV.
         :param task_type: The target task type in relation to the data being used.
         :param csv_config: The CSV file format of the raw data.
         :param excel_config: The Excel file format of the raw data.
@@ -451,37 +489,42 @@ class Modela:
     def Datasets(self) -> DatasetClient:
         return self.__dataset_client
 
-    def Dataset(self, namespace="", name="", gen_datasource: bool = False, version=Resource.DefaultVersion,
-                target_column: str = None, datasource: Union[DataSource, str] = "", bucket: str = "default-minio-bucket",
+    def Dataset(self, namespace="", name="", version=Resource.DefaultVersion, gen_datasource: bool = False,
+                lab: Union[ObjectReference, Lab, str] = "default-lab",
+                target_column: str = None, datasource: Union[DataSource, str] = "",
+                bucket: Union[VirtualBucket, str] = "default-minio-bucket",
                 dataframe: pandas.DataFrame = None, data_file: str = None, data_bytes: bytes = None,
-                workload: Workload = Workload("general-large"), sample: SampleSettings = None, task_type: TaskType = None,
-                notification: NotificationSetting = None) -> Dataset:
+                workload: Workload = Workload("general-large"), fast: bool = False,
+                sample: SampleSettings = None, task_type: TaskType = None,
+                notification: NotificationSettings = None) -> Dataset:
         """
         Fetch or create a new Dataset resource
 
+        :param client: The Dataset client repository, which can be obtained through an instance of Modela.
         :param namespace: The target namespace of the resource.
         :param name: The name of the resource.
-        :param version: The version of the resource.
+        :param lab: The object reference, Lab object, or lab name under the tenant of the resource for which all
+            Dataset-related workloads will be performed under.
         :param gen_datasource: If true, a Datasource resource will be created from the uploaded dataset and applied to
             the Dataset resource.
         :param target_column: If gen_datasource is enabled, then the target column of the data source must be specified.
-        :param bucket: The bucket which the raw dataset data will be uploaded to.
         :param datasource: If specified as a string, the SDK will attempt to find a Data Source resource with the given name.
-            If specified as a Data Source object, or if one was found with the given name, it will be applied to the Dataset.
-        :param dataframe: If specified, the Pandas Dataframe will be serialized and uploaded for ingestion with the Dataset resource.
-        :param data_file: If specified, the SDK will attempt read a file with the given path and will upload the
-            contents of the file for ingestion with the Dataset resource.
-        :param data_bytes: If specified, the SDK will upload the given raw data for ingestion with the Dataset resource.
+            If specified as a Data Source, or if one was found with the given name, it will be applied to the Dataset.
+        :param bucket: The bucket which the raw dataset data will be uploaded to.
+        :param dataframe: The Pandas Dataframe will be serialized and uploaded for ingestion with the Dataset resource.
+        :param data_file: The file path which will be read and uploaded for ingestion with the Dataset resource.
+        :param data_bytes: The raw data as bytes that will be uploaded for  ingestion with the Dataset resource.
         :param workload: The resource requirements which will be allocated for Dataset ingestion.
-        :param sample: The sample settings of the dataset, which if enabled will ingest a Dataset with a portion of the uploaded data.
+        :param fast: If enabled, the Dataset will skip validation, profiling, and reporting.
+        :param sample: The sample settings of the dataset, which if enabled will select only a portion of the Dataset
+          for further processing.
         :param task_type: The target task type in relation to the data being used.
         :param notification: The notification settings, which if enabled will forward events about this resource to a notifier.
         """
-        return Dataset(MDDataset(), self.Datasets, namespace, name, version, gen_datasource, target_column,
-                       datasource, bucket, dataframe, data_file, data_bytes, workload,
+        return Dataset(MDDataset(), self.Datasets, namespace, name, version, lab, gen_datasource, target_column,
+                       datasource, bucket, dataframe, data_file, data_bytes, workload, fast,
                        sample, task_type, notification)
 
-
     @property
     def FileService(self) -> FileService:
         return self.__fileservice_client
@@ -616,8 +659,34 @@ class Modela:
     def Predictors(self) -> PredictorClient:
         return self.__predictor_client
 
-    def Predictor(self, namespace="", name="") -> Predictor:
-        return Predictor(MDPredictor(), self.Predictors, namespace, name)
+    def Predictor(self, namespace="", name="", version=Resource.DefaultVersion,
+                  serving_site: Union[ObjectReference, ServingSite, str] = "default-serving-site",
+                  model: Union[Model, str] = None, models: List[ModelDeploymentSpec] = None, port: int = 3000,
+                  path: str = None, access_type: AccessType = None, replicas: int = 0, autoscale: AutoScalingSpec = None,
+                  workload: Workload = None) -> Predictor:
+        """
+        :param namespace: The target namespace of the resource.
+        :param name: The name of the resource.
+        :param version: The version of the resource.
+        :param serving_site: The object reference, Serving Site object, or name under the tenant of the resource for which the
+            predictor will be deployed under.
+        :param model: The Model object or name that the predictor will serve predictions for. This parameter
+            will instantiate the predictor with a singe default deployment specification for the model.
+        :param models: The list of model deployment specifications that will be applied to the predictor.
+        :param port: The port of the predictor, which if applicable will expose the service internally or externally.
+        :param path: The path that the predictor will expose a service on. If this parameter is not specified, then
+            a path will be generated based on the access type and predictor name.
+        :param access_type: The access type of the Predictor. See https://www.modela.ai/docs/docs/serving/production/
+            for documentation on how different access types expose the Predictor service
+        :param replicas: The amount of replicas for the predictor, which if greater than zero will serve the prediction
+            service on multiple pods.
+        :param autoscale: If set to true, the predictor's deployment will scale based on the amount of incoming traffic
+            to the service.
+        :param workload: The workload specification which determines the resources which will be allocated to the
+            prediction service.
+        """
+        return Predictor(MDPredictor(), self.Predictors, namespace, name, version, serving_site, model, models,
+                         port, path, access_type, replicas, autoscale, workload)
 
     @property
     def Alerts(self) -> AlertClient:
@@ -717,6 +786,18 @@ class Modela:
     def VirtualVolume(self, namespace="", name="") -> VirtualVolume:
         return VirtualVolume(MDVirtualVolume(), self.VirtualVolumes, namespace, name)
 
+    @property
+    def UserRoleClasses(self) -> UserRoleClassClient:
+        return self.__userroleclass_client
+
+    def UserRoleClass(self, namespace="", name="", rules: List[Rule] = None) -> UserRoleClass:
+        """
+        :param namespace: The target namespace of the resource.
+        :param name: The name of the resource.
+        :param rules: The list of rules which any Account associated with the User Role Class resource may perform.
+        """
+        return UserRoleClass(MDUserRoleClass(), self.UserRoleClasses, namespace, name, rules)
+
     @property
     def Meetings(self) -> MeetingClient:
         return self.__meeting_client
@@ -763,8 +844,47 @@ class Modela:
     def ModelAutobuilders(self) -> ModelAutobuilderClient:
         return self.__modelautobuilder_client
 
-    def ModelAutobuilder(self, namespace="", name="") -> ModelAutobuilder:
-        return ModelAutobuilder(MDModelAutobuilder(), self.ModelAutobuilders, namespace, name)
+    def ModelAutobuilder(self, namespace="", name="", version=Resource.DefaultVersion,
+                         lab: Union[ObjectReference, Lab, str] = "default-lab",
+                         serving_site: Union[ObjectReference, ServingSite, str] = "default-serving-site",
+                         task_type: TaskType = TaskType.BinaryClassification, workload: Workload = Workload("general-large"),
+                         bucket: Union[ObjectReference, VirtualBucket, str] = "default-minio-bucket",
+                         dataframe: pandas.DataFrame = None, data_file: str = None, data_bytes: bytes = None,
+                         target_column: str = None, objective: Metric = Metric.Accuracy,
+                         feature_selection: bool = False, feature_engineering: bool = False, max_models: int = 1,
+                         max_time: int = 512, trainers: int = 1, predictor_access_type: AccessType = AccessType.ClusterIP,
+                         predictor_autoscale: bool = False, create_data_app: bool = False) -> ModelAutobuilder:
+        """
+        :param namespace: The target namespace of the resource.
+        :param name: The name of the resource.
+        :param version: The version of the resource.
+        :param lab: The object reference, Lab object, or lab name under the tenant of the resource for which all
+            data science workloads will be performed under.
+        :param serving_site: The object reference, Serving Site object, or name under the tenant of the resource for which
+            the created Predictor will be deployed under.
+        :param task_type: The target task type in relation to the data being used.
+        :param workload: The workload specification which determines the resources which will be allocated for training
+            and serving workloads.
+        :param bucket: The bucket which the raw dataset data will be uploaded to.
+        :param dataframe: The Pandas Dataframe will be serialized and uploaded to create a new Dataset resource.
+        :param data_file: The file path which will be read and uploaded to create a new Dataset resource.
+        :param data_bytes: The raw data as bytes that will be uploaded to create a new Dataset resource.
+        :param target_column: The target column of the Dataset to be created.
+        :param objective: The objective metric to be used when determining the best model from the Study.
+        :param feature_selection: If True feature selection will be performed on the Dataset before the model search.
+        :param feature_engineering: If True feature engineering will be performed on the Dataset before the model search.
+        :param max_models: The maximum number of models to sample before selecting the best model.
+        :param max_time: The maximum amount of time in seconds which the model search can run for.
+        :param trainers: The number of parallel trainers to allocate during the model search
+        :param predictor_access_type: The access type of the Predictor to be created. See https://www.modela.ai/docs/docs/serving/production/
+            for documentation on how different access types expose the Predictor service
+        :param predictor_autoscale: If true the created Predictor will automatically scale for traffic
+        :param create_data_app: If true a Data Application will be created to serve a live model dashboard
+        """
+        return ModelAutobuilder(MDModelAutobuilder(), self.ModelAutobuilders, namespace, name, version, lab,
+                                serving_site, task_type, workload, bucket, dataframe, data_file, data_bytes,
+                                target_column, objective, feature_selection, feature_engineering, max_models,
+                                max_time, trainers, predictor_access_type, predictor_autoscale, create_data_app)
 
     @property
     def ModelCompilerRuns(self) -> ModelCompilerRunClient:
@@ -827,22 +947,24 @@ class Modela:
         return self.__study_client
 
     def Study(self, namespace="", name="", version=Resource.DefaultVersion, dataset: Union[str, Dataset] = "",
+              test_dataset: Union[str, Dataset] = "",
               lab: Union[ObjectReference, Lab, str] = "default-lab", bucket: Union[VirtualBucket, str] = None,
-              task_type: TaskType = None, objective: Metric = None, search: ModelSearch = None,
+              objective: Metric = Metric.Accuracy, search: ModelSearch = None,
               fe_search: FeatureEngineeringSearch = None, baseline: BaselineSettings = None,
               ensemble: Ensemble = None, trainer_template: Training = None, interpretability: Interpretability = None,
-              schedule: StudySchedule = None, notification: NotificationSetting = None, garbage_collect: bool = True,
-              keep_best_models: bool = True, timeout: int = 600, template: bool = False) -> Study:
+              schedule: StudySchedule = None, notification: NotificationSettings = None, garbage_collect: bool = True,
+              keep_best_models: bool = True, fast: bool = False, timeout: int = 600, template: bool = False) -> Study:
         """
         :param namespace: The target namespace of the resource.
         :param name: The name of the resource.
         :param version: The version of the resource.
         :param dataset: If specified as a string, the SDK will attempt to find a Dataset resource with the given name.
             If specified as a Dataset object, or if one was found with the given name, it will be used in the Study.
+        :param test_dataset: If specified, the Dataset will be used as the test dataset and the Dataset specified by
+            the `dataset` field will be used solely as the training dataset
         :param lab: The object reference, Lab object, or lab name under the default-tenant for which all Study-related
             workloads will be performed under.
         :param bucket: The Bucket object or name of the bucket which will store the Study artifacts
-        :param task_type: The ML task type of the Study
         :param objective: The objective metric relevant to the task type.
         :param search: The search parameters define how many models to sample
         :param fe_search: The feature engineering search parameters of the Study
@@ -851,22 +973,26 @@ class Modela:
         :param ensemble: The ensemble settings for the Study, which if enabled will combine the top estimators of
             the study after the initial model search.
         :param trainer_template: The training template for each model created by the Study.
-        :param interpretability: The interpretability settings for the Study, which when enabled can produce ICE, LIME,
-            and Shap value plots
-        :param schedule: The schedule for the study to run chronically
+        :param interpretability: The interpretability configuration for the Study, which specifies what type of
+            model explainability plots will be generated from Shap values if they are computed
+        :param schedule: The schedule for the study to run chronically.
         :param notification: The notification settings, which if enabled will forward events about this resource to a notifier.
         :param garbage_collect: If enabled, models which did not move past the testing stage will be garbage collected by
             the system.
         :param keep_best_models: If enabled, the best models from each algorithm will not be garbage collected.
+        :param fast: If enabled, the Study will skip profiling, explaining, and reporting.
         :param timeout: The timeout in seconds after which the Study will fail.
         :param template: If the Study is a template it will not start a search and can only be used as a template for
             other studies.
         """
-        return Study(MDStudy(), self.Studies, namespace, name, version, dataset, lab, bucket, task_type, objective,
+        return Study(MDStudy(), self.Studies, namespace, name, version, dataset, test_dataset, lab, bucket, objective,
                      search, fe_search, baseline, ensemble, trainer_template, interpretability, schedule,
-                     notification, garbage_collect, keep_best_models, timeout, template)
+                     notification, garbage_collect, keep_best_models, fast, timeout, template)
 
     def close(self):
+        if hasattr(self, 'pf_process'):
+            self.pf_process.kill()
+
         if self._channel:
             self._channel.close()
 
